#ifndef H_ASTEROID_SPAWNING_SUBSYSTEM
#define H_ASTEROID_SPAWNING_SUBSYSTEM

#include "Asteroid.h"
#include "Preferences.h"
#include "Random.h"
#include "Point.h"

class AsteroidSpawningSubsystem{
    public:

    /**
      constructor
      */
    AsteroidSpawningSubsystem();
    /**
      inform spawner about passed time
      */
    void updateTime(double timeDelta);

    /**
      init spawner - it will automatically
      read /asteroids/ section from
      preferences
      */
    void init(Preferences& preferences);

    /**
      return number of asteroids pending to spawn
      */
    int getSpawnCount();

    /**
      spawn new asteroid
      */
    Asteroid* spawn(Point position);

    private:
    /**
      how long takes new asteroid to spawn
      */
    double spawnTime;

    /**
      spawnTime's variability
      */
    double spawnTimeVariability;

    /**
      time when next spawning will occur
      */
    double nextEventTime;

    /**
      actual time
      */
    double actualTime;





    /**
      initial angular velocity of asteroid,
      direction is random
      */
    double angularVelocity;

    /**
      angularVelocity's variability
      */
    double angularVelocityVariability;

    /**
      initial velocity of asteroid,
      direction is asteroid's angle
      */
    double velocity;

    /**
      velocity's variability
      */
    double velocityVariability;

    double radius;
    double radiusVariability;


    /**
      number of asteroids pending to spawn
      */
    int asteroidsToSpawn;
};

#endif
