#ifndef H_EDGE_COLLIDER
#define H_EDGE_COLLIDER
#include "MassObject.h"

class EdgeCollider{
    public:

        /**
          constructor
          */
        EdgeCollider();

        /**
          initialize, read from preferences
          */
        void initialize(const int width, const int height);

        /**
          collide mass objects with edges
          */
        // {{{ collideMassObjects
        template <typename InputIterator>
        void collideMassObjects(InputIterator begin, InputIterator end){
            while (begin!=end){
                computeEdgeCollisions(*begin);
                ++begin;
            }
        }
        // }}}

        // {{{ edgeKillMassObjects
        template <typename InputIterator>
        void edgeKillMassObjects(InputIterator begin, InputIterator end){
            while (begin!=end){
                edgeKill(*begin);
                ++begin;
            }
        }
        // }}}

    private:
    /**
      called from computeEdgeCollisions for each MassObject
      */
    void computeEdgeCollisions(MassObject* object);
    void edgeKill(MassObject* object);

    /**
      distance, at which affectors starts to apply force
      */
    int edgeDistance;

    /**
      actual force used to affect object,
      note that force_object=edgeForce*d^2
      where d is distance from edge
      */
    double edgeForce;

    /**
      also, we not only push to object, but move them back
      (addying instant impact)
      */
    double edgeImpact;

    int width;
    int height;

    double killDistance;
};

#endif
