#ifndef H_PLASMA_WEAPON_Slot
#define H_PLASMA_WEAPON_Slot
#include "WeaponSlot.h"

class PlasmaWeaponSlot: public WeaponSlot {
    public:
        /**
          return reloading time of weapon.
          reloading time is for 1 shoot.
          note that cannonBall counts as more shoots
          */
        virtual double getReloadTime();

        /**
          return minimal time to wait, until next shoot
          is allowed
          */
        virtual double getTimeToNextShoot();

        /**
          return actual number of shoot that can be shoot,
          note that this number>0 do not mean that you can shoot
          - also getTimeToNextShoot() must be zero!
          */
        virtual double getNumberOfRemainingShoots();

        /**
          return maximum energy of WeaponLoaders,
          */
        virtual double getMaxEnergy();

        /**
          return energy needed to shoot
          */
        virtual double getShootEnergy();

        /**
          return actual state of weapon energy.
          note that it is incremented by 1energy/second
          */
        virtual double getActualEnergy();

        /**
          get minimum delta angle to shoot
          one can shoot in range (getMinAngle()+actualAngle,
          getMaxAngle()+actualAngle)
          */
        virtual double getMinAngle();

        /**
          @see getMinAngle()
          get maximum delta angle to shoot
          */
        virtual double getMaxAngle();

        /**
          get minimal speed of projectile upon shooting
          note that this value is relative to ship's speed
          */
        virtual double getMinSpeed();

        /**          
          get maximal speed of projectile upon shooting
          note that this value is relative to ship's speed
          */
        virtual double getMaxSpeed();

        /**
          initialize all static variables
          */
    static void initialize();

    /**
      constructor, clears internal variables
      */
    PlasmaWeaponSlot();

    /**
      destructor
      */
    virtual ~PlasmaWeaponSlot();

    /**
      notify slot of new time quantum
      */
    virtual void timeNotify(double timeDelta);

    /**
      player wants to shoot - check if it can and then
      shoot
      */
    virtual void shoot(Ship* ship, double angle, double speed,
             double additionalInfo, World* world);

    /**
      make copy, internal use in ship's copy
      */
    virtual WeaponSlot* clone();

    /**
      convert to bytestream
      */
    virtual void serialize(std::vector<unsigned char>& destination);

    /**
      convert bytestream to PlasmaWeaponSlot, note that the marker
      must be removed before
      */
    static PlasmaWeaponSlot* deserialize(std::vector<unsigned
            char>::iterator& it);

    private:
      /**
        actual time to reload weapon
        */
      double reload;

      /**
        actual energy
        */
      double energy;
    public:

      /**
        time to realod after shoot
        */
      static double reloadTime;

      /**
        maximum energy weaponSlot can have,
        this determines maximum number of fast shoots
        */
      static double maximumEnergy;

      /**
        energy needed to shoot
        */
      static double shootEnergy;

      /**
         minimum relative angle to ship's angle
         */
      static double minAngle;

      /**
        maximum relative angle to ship's angle
        */
      static double maxAngle;

      /**
        minimum relative speed to ship's speed
        */
      static double minSpeed;

      /**
        maximum relative speed to ship's speed
        */
      static double maxSpeed;
};

#endif
