#ifndef H_SERIALIZER
#define H_SERIALIZER

#include "Macro.h"
#include "Errors.h"
#include "Assert.h"
#include <vector>
#include "BasicObject.h"
#include "Point.h"

class Serializer{
    public:
    /**
      check if we are on the compatible architecture
      */
    static void checkTypes();

    static const int charSize=1;
    static const int intSize=4;
    static const int doubleSize=8;
    static const int floatSize=4;
    static const int MARKER=47;

    static void serializeChar(std::vector<unsigned
            char>&destination,const char value);

    static char deserializeChar(std::vector<unsigned char>::iterator& it);

    static void serializeInt(std::vector<unsigned
            char>&destination,const int value);

    static int deserializeInt(std::vector<unsigned char>::iterator& it);

    static void serializeDouble(std::vector<unsigned
            char>&destination,const double value);

    static double deserializeDouble(std::vector<unsigned
            char>::iterator& it);

    static void serializeFloat(std::vector<unsigned
            char>&destination,const float value);

    static float deserializeFloat(std::vector<unsigned
            char>::iterator& it);

    static void serializePoint(std::vector<unsigned char>&
            destination, const Point value);

    static Point deserializePoint(std::vector<unsigned
            char>::iterator& it);

    template <typename Vector>
        static void serializeObjects(
                std::vector<unsigned char>& destination,
                Vector objects) {
            serializeInt(destination,objects.size());
            FOREACH(it,objects) {
                (*it)->serialize(destination);
            }
            serializeInt(destination,MARKER);
        }

    template <typename Object>
        static void deserializeObjects(
                std::vector<unsigned char>::iterator& it,
                std::vector<Object*>& objects){
            assert(objects.size()==0,"Trying to deserialize "
                    "to non-empty vector, memory leaks possible");

            int cnt=deserializeInt(it);
            FOR(q,cnt) {
                
                Object* ptr=dynamic_cast<Object*>(
                    deserializeObject(it));
                assert(ptr!=NULL,"dynamic cast failed - "
                        "serialization data corrupted");
                        
                objects.push_back(ptr);
            }
            int marker=deserializeInt(it);
            assert(marker==MARKER,"marker check failed, data "
                    "corrupted in deserialization");
        }

  static  BasicObject* deserializeObject(
            std::vector<unsigned char>::iterator& it);
};
#endif
