#ifndef H_UTILS
#define H_UTILS


#include "Ship.h"

/**
  class, or struct representing
  distance & time.
  Used by some functions 
  as return argument
  */
class DistanceTime {
    public:
     double distance;
     double time;
};

/**
  utilities, common solutions for geometry
  */
class Utils{
    public:
 /**
   standard way to navigate to point:

   Ship* ptr=myships[i];
   Point target=Point(200,200);

   if (ptr->position - target).length()<10) then onThePosition :-)


   double a=getShipAccelerationJets(ptr,target);
   response[i].accelerationJets=a;
   int isAcceleratinng=a>0;

   double angle=getObjectToTargetAngleWithRespectToVelocity(ptr,target
     isAcceleratinng);

   response[i].rotationJets=getShipRotationalJets(ptr,angle);
   */
 static void shipNavigate(Ship* ship, PlayerResponse* response,
         Point target);

 static void shipNavigate(Ship& ship, PlayerResponse& response,
         Point target);

 /**
   calculate effective way of using ship's jets to obtain desired
   angle (with no spin afterwards) in minimum time
   note that if you want to have spin after rotation, you can't
   use this function.
   */
 static double getShipRotationalJets(Ship* ship, double angle);

 /**
   calculate effective way of using ship's jets to move to target.
   Note that this function assumes, that you already have
   right angle. Complex function maybe will be added later
   */

 static double getShipAccelerationJets(Ship* ship, Point target);

 /**
   this function calcutale correction to angle to navigate to target
   */
 static double getObjectToTargetAngleWithRespectToVelocity(MassObject*
         object, Point target, int isActuallyAccelerating,double
         weight);

 static double getMassObjectRotationalJets(MassObject* mobject, double angle,double acceleration);

 /**
   return angle normalized to interval (-Pi,Pi>
   note that if abs(angle)>10^9, function returns 0
   (as it fails - too low accuracy)
   */
 static double normalizeAngle(double angle);

 /**
   returns distance of two objects
   note that returning distance of two points a,b can be
   done as (a-b).length()
   */
 static double distance(BasicObject *a,BasicObject *b);


 /**
   return angle of Point Targer as observed from vPoint Observed
   (0,0),(1,0) returns 0
   (0,0),(0,1) returns pi/2
   (0,0),(-1,0) returns pi
   (0,0),(0,-1) returns 3/2 pi
   */
 static double getAngle(Point observer, Point target);

 /**
   return angle composed from three points, b is middle point
   note that this angle is normalized ie in range (-Pi,Pi>
   */
 static double getAngle(Point a,Point b,Point c);

 /**
   @see getAngle(Point observer, Point targert)
   same as Points only done on object's positions
   */
 static double getAngle(BasicObject *observer, BasicObject* target);

 /**
   get relative angle of object target to observer,
   note that observer's angle is taken into account
   */
 static double getRelativeAngle(BasicObject* observer, BasicObject*
         target);

 /**
   return position of Point position if it is moving with velocity
   and constant acceleration after time
   */
 static Point getPosition(Point position, Point velocity, Point
         acceleration, double time);

 /**
   same as with Points
   */
 static Point getPosition(MassObject* object, Point acceleration,
         double time);

 /**
   return position of object moving with velocity and no acceleration
   after time time
   */
 static Point getPosition(MassObject* object, double time);

 /**
   we have two points at positions pos1, pos2 moving with velocities
   vel1, vel2. Question is, in what (non-negative) time they
   will be at smallest distance.
   */
 static DistanceTime getMinimalDistance(Point pos1, Point vel1, Point pos2,
         Point vel2);

 /**
   @see getMinimalDistance for Points
   */
 static DistanceTime getMinimalDistance(MassObject* object1, MassObject*
         object2);


 /** 
   get first (non-negative) time, when objects determined by positions pos and
   velocities vel will be at distance smaller or equal to distance.
   Note that this function returns -1 on fail, i.e. if objects newer
   be at such a distance
   */
 static double getTimeToDistance(Point pos1, Point vel1, Point pos2,
         Point vel2, double distance);

 /**
   @see getTimeToDistance for Points
   */
 static double getTimeToDistance(MassObject* object1, MassObject*
         object2, double distance);



};

#endif
