#include <iostream>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <errno.h>

#include "Proboj.h"
#include "Macro.h"


#include "Errors.h"
#include "Exceptions.h"
#include "Graphics.h"
#include "Preferences.h"

#include "World.h"
#include <cmath>
#include "WorldArrays.h"
#include "Utils.h"
#include "ObjectGraphics.h"

#define FD 47

#define LOG_FILE "player.log"
#define INI_FILE "proboj.ini"
#define PERFORMANCE_LOOPS 500
#define MAX_LOOPS 25000


int performanceTimer;
int performanceDelta;
int performanceCounter;
int mouseX;
int mouseY;
int frames=0;

int fd;

void think(WorldArrays &world,std::vector<PlayerResponse> &response);
void init_game();

World world,tmpWorld;
Surface surface;
PlayerResponse playerResponse;
double angle=0;
int player=0;

//  {{{ init
int init(){
    playerResponse.fireWeapon=-1;
    
    Errors::createInstance(LOG_FILE);
    Preferences::createInstance();
    Preferences::_loadFromFile(INI_FILE);
    world.initialize("mapa.dat");
    init_game();
 
    return 1;

}
// }}}

// {{{ finalize
void finalize(){
}
// }}}


 // {{{ main loop doLoop
int doLoop(){
    std::vector<unsigned char> tmp;
    //receivnem
    tmp.resize(sizeof(int));
//    std::cout<<"Hrac ide receivovat\n";
    int ret=::recv(fd,&tmp[0],sizeof(int),MSG_PEEK);
//    std::cout<<"Hrac nieco dostal: "<<ret<<"\n";
    if(ret==-1)return 0;
    int res=(*(int*)&tmp[0]);
    tmp.resize(res);
//    std::cout<<"Packet ma velkost :"<<tmp.size()<<std::endl;
    unsigned int dt=0;
    while(dt<tmp.size()){
        ret=::recv(fd,&tmp[dt],res-dt,0);
	dt+=ret;
//    std::cout<<errno<<" "<<strerror(errno)<<std::endl;
//    std::cout<<"Hrac opat nieco dostal, velkost je: "<<ret<<" bajtof\n";
	if(ret==-1)return 0;
    }
//    std::cout<<"Hrac uspesne dostal cely world\n";
   world.serialize(tmp);
    std::vector<unsigned char>::iterator it=tmp.begin()+sizeof(int);
//    std::cout<<"1\n";
    world.deserialize(it);
//    std::cout<<"2\n";
    WorldArrays worldArrays;
//    std::cout<<"3\n";
    worldArrays.clearAll();
 //   std::cout<<"4\n";
    world.prepareDataForPlayer(player,&worldArrays);
//    std::cout<<"5\n";
    std::vector<PlayerResponse> response(worldArrays.myShips.size());
//    std::cout<<"Hrac rozmysla\n";
    think(worldArrays,response);
//    std::cout<<"Hrac dorozmyslal\n";
    //sendnem
    tmp.resize(sizeof(int));
    for(unsigned int i=0;i<response.size();i++)
	response[i].serialize(tmp);
    *(int*)&tmp[0]=tmp.size();
//    std::cout<<"Hrac ide poslat packet\n";
    dt=0;
    while(dt<tmp.size()){
        int ret=send(fd,&tmp[dt],tmp.size()-dt,MSG_NOSIGNAL);
	if(ret==-1)return 1;
	dt+=ret;
    }
//    std::cout<<"Hrac poslal packet, dlzka by mala byt: "<<tmp.size()<<"bajtof\n";

    return 1;

}
// }}}

// {{{ main
int main(int argc, char *argv[]){
    std::string type="local";
    std::string host="0.0.0.0";
    int port=0;
    int i=1;
    while(i<argc){
	    if(!strcmp(argv[i],"-t")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    type=std::string(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-h")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    host=std::string(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-p")){
		    if(i+1>=argc){
			    std::cerr<<"Bat parameters\n";
			    return 1;
		    }
		    port=atoi(argv[i+1]);
		    i+=2;
	    }else if(!strcmp(argv[i],"-n")){
		if(i+1>=argc){
		    std::cerr<<"Bat parameters\n";
		    return 1;
		}
		player=atoi(argv[i+1]);
		i+=2;
	    }else {
		    std::cerr<<"Bat parameters\n";
		    return 1;
	    }
    }
    if(type=="local"){
	    fd=FD;
    }else if(type=="remote"){
	    struct sockaddr_in addr;
	    if (inet_pton(AF_INET, host.c_str(), &addr.sin_addr.s_addr) == -1) {
		    std::cerr<<"inet_pton error: "<<strerror(errno)<<std::endl;
		    return 1;
	    }
	    addr.sin_port = htons(port);
	    addr.sin_family = AF_INET;
	    fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	    if (fd == -1) {
		    std::cerr<<"Nepodarilo sa otvorit socket "<<strerror(errno)<<std::endl;
		    return 1;
	    }
	    if (::connect(fd, (struct sockaddr*)&addr, sizeof(addr))) {
		    std::cerr<<"Nepodaril sa connect "<<strerror(errno)<<std::endl;
		    return 1;
	    }
    }else {
	    std::cerr<<"Bat type\n";
	    return 0;
    }
    std::cout<<"Hrac nastartoval\n";
    try {
        if (init()) {
            while (doLoop());
        }

        finalize();
    } catch (Exception &e){
	shutdown(fd,SHUT_RDWR);
	close(fd);
        printf("There was an exception %s %s\n",
                e.what(),e.getStringStack().c_str());
    } catch (std::exception &e){
	shutdown(fd,SHUT_RDWR);
	close(fd);
        printf("There was an exception %s\n",e.what());
    }
shutdown(fd,SHUT_RDWR);
close(fd);


}

// }}}
