#include "player.h"
#include "Utils.h"

#include <algorithm>

using namespace std;

//using namespace Utils;

void init_game() {

}

#define MAXTIME 20
#define MAP_WIDTH 800
#define MAP_HEIGHT 600

#define TIME_STEP 0.3

double Map[MAXTIME][MAP_WIDTH][MAP_HEIGHT];
//Ship lastShip[10];

Ship findTarget(Ship &ship, WorldArrays &world) {
	//Ship shipp;
	if(world.otherShips.size()>0){
	Ship nearest = world.otherShips[0];
	for (int i = 1; i < world.otherShips.size(); i++)
		if ( (ship.position - world.otherShips[i].position ).length() < (ship.position - nearest.position ).length() )
			nearest = world.otherShips[i];
		return nearest;
	} else {
		Ship newShip;
		return newShip;
	}
}

Point precomputePosition(Ship targetShip, double distance, double velocity){
	Point p = Utils::getPosition(&targetShip, distance / velocity);
	return p;
}

double AngleDelta(double A, double B) {
	A = Utils::normalizeAngle(A);
	B = Utils::normalizeAngle(B);
	return min( abs(A - B) , 2 * M_PI - abs(B - A) );
}

bool DetectRocket(Ship ship, WorldArrays &world) {
	for (int i = 0; i < world.missile.size(); i++) {
		double missileDistance = (world.missile[i].position - ship.position).length();
		double missileAngle = Utils::getAngle(ship.position, world.missile[i].position);
		if (missileDistance < 100 && AngleDelta(missileAngle, ship.angle) < 0.8) {
			return true;
		}
	}
	return false;
}

bool FriendlyFire(Ship ship, int weapon, double targetDistance, WorldArrays &world) {
	double delta = M_PI/20;
	if (weapon==2)
		delta = M_PI/6;
	
	for (int i = 0; i < world.myShips.size(); i++) {
		double friendDistance = (world.myShips[i].position - ship.position).length();
		double friendyAngle = Utils::getAngle(ship.position, world.myShips[i].position);
		if (friendDistance < targetDistance && friendDistance > 1.0 && AngleDelta(friendyAngle, ship.angle) < delta)
			return true;
	}
	return false;
}

#define PLASMA_RADIUS 100

int blocked = 0;
int total = 0;

bool SecureFire(Ship ship, WorldArrays &world) {
	total++;
	for (int i = 0; i < world.plasmaBalls.size(); i++) {
		double ballDistance = (world.plasmaBalls[i].position - ship.position).length();
		double plasmaAngle = Utils::getAngle(ship.position, world.plasmaBalls[i].position);
		if (ballDistance < PLASMA_RADIUS && AngleDelta(plasmaAngle, ship.angle) < 0.8) {
			blocked++;
			return false;
		}
		if (ballDistance < 25) return false;
	}
	return true;
}


void think(WorldArrays &world,std::vector<PlayerResponse> &response){

//	precompute map for 1.0 seconds
/*
	for (int i = 0; i < world.plasmaBalls.size(); i++) {
		for (int t = 0; t < MAXTIME; t++) {
			Point P = Utils::getPosition( &world.plasmaBalls[i] , TIME_STEP * t);
			addMapToPoint( P, t, -100 );
		}
	}
*/
	
	for (int j = 0; j < world.myShips.size(); j++) {
	//	printf("------ next for ship %d -----------------------\n", j);
		
		Ship ship = world.myShips[j];
		PlayerResponse &playerResponse=response[j];
		
		Ship TargetShip = findTarget(world.myShips[j], world);
		//lastShip[j]=TargetShip;
		double distance = (TargetShip.position - ship.position).length();
		
		double angleToShip = Utils::getAngle(ship.position, TargetShip.position);
		
		playerResponse.fireWeapon = -1;
		
		bool normalNavigation = true;
		
		/* //autodestruction
		if (distance < 25 && ship.live <= 20) {
			playerResponse.fireWeapon = -2;
			playerResponse.weaponSpeed= ship.weaponSlots[1]->getMaxSpeed();
			playerResponse.weaponAngle = 0;
			playerResponse.weaponAdditional = 6;
			normalNavigation = false;
		}*/
		
		if (!normalNavigation) continue;

		if (world.blackHoles.size()>0)
		for (int i = 0; i < world.blackHoles.size(); i++) {
			BlackHole hole = world.blackHoles[i];
			double time = Utils::getTimeToDistance(ship.position, ship.velocity, hole.position, hole.velocity, hole.radius * 1.3);
			double distance = (ship.position - hole.position).length();
			if ((time < 3.0 && time > 0.0) || distance < hole.radius * 2.5) {
				normalNavigation = false;
				Point delta = (ship.position - hole.position) * 100;
				Point point = ship.position + delta;
				Utils::shipNavigate(&ship, &playerResponse, point);
				printf("ALARM! - black hole \n");
			}
		}
	/*	if ((DetectRocket(ship,world)) && (!FriendlyFire(ship,1,200,world)) && (SecureFire(ship,world))) {
			playerResponse.fireWeapon = 1;
			playerResponse.weaponSpeed= ship.weaponSlots[1]->getMaxSpeed();
			playerResponse.weaponAngle = 0;
			playerResponse.weaponAdditional = 6;
		}*/
	//	printf("nase zrychlenie: %lf\n", ship.acceleration.length());
	//	printf("blokovanie %lf\n", (double)blocked/total);
		
		Point Target = precomputePosition(TargetShip, distance, world.myShips[j].weaponSlots[1]->getMaxSpeed());

	//	double angleToTarget = Utils::getAngle(ship.position, Target);
	
		if (!normalNavigation) continue;
		
		if (distance <= 400.0) { // attack
			if (distance < 400) playerResponse.accelerationJets=distance/800;
			if ((ship.weaponSlots[1]->getTimeToNextShoot() <= 0.01) && (SecureFire(ship,world))) {
				Point ShotVector = ship.velocity;
				Point lookVector = Point(ship.angle);
				lookVector *= ship.weaponSlots[1]->getMaxSpeed();
				ShotVector += lookVector;
				
				double timeToHit = Utils::getTimeToDistance(ship.position, ShotVector, TargetShip.position, TargetShip.velocity, 60.0);

				if ((timeToHit >= 0.0) && (!FriendlyFire(ship,1,distance,world))) {
					playerResponse.fireWeapon = 1;
					playerResponse.weaponSpeed= ship.weaponSlots[1]->getMaxSpeed();
					playerResponse.weaponAngle = 0;
					playerResponse.weaponAdditional = 6;
				}
				
			} else if ((ship.weaponSlots[0]->getTimeToNextShoot() <= 0.01) && (SecureFire(ship,world) && !(rand()%2))) {
				Point ShotVector = ship.velocity;
				Point lookVector = Point(ship.angle);
				lookVector *= ship.weaponSlots[0]->getMaxSpeed();
				ShotVector += lookVector;
				
				double timeToHit = Utils::getTimeToDistance(ship.position, ShotVector, TargetShip.position, TargetShip.velocity, 60.0);
				
				if (timeToHit >= 0.0 && !FriendlyFire(ship,0,distance,world)) {
					playerResponse.fireWeapon = 0;
					playerResponse.weaponSpeed= ship.weaponSlots[0]->getMaxSpeed();
					playerResponse.weaponAngle = 0;
					playerResponse.weaponAdditional = 6;
				}
			} else if ((AngleDelta(ship.angle, angleToShip) < M_PI/12) && (!FriendlyFire(ship,2,distance,world))&& (SecureFire(ship,world))) {
				playerResponse.fireWeapon = 2;
				playerResponse.weaponSpeed= ship.weaponSlots[2]->getMaxSpeed();
				playerResponse.weaponAngle = 0;
				playerResponse.weaponAdditional = 0;
			}
			playerResponse.rotationJets = Utils::getShipRotationalJets(&ship, angleToShip);
		} else { // move
			if ((AngleDelta(ship.angle, angleToShip) < M_PI/16) && (!FriendlyFire(ship,2,distance,world)) && (SecureFire(ship,world))) {
				playerResponse.fireWeapon = 2;
				playerResponse.weaponSpeed= ship.weaponSlots[2]->getMaxSpeed();
				playerResponse.weaponAngle = 0;
				playerResponse.weaponAdditional = 0;
			}
			
			Utils::shipNavigate(&ship, &playerResponse, Target);//TargertShip.position
		}
		//if (!SecureFire(ship, world)) playerResponse.accelerationJets=0;	
		
		
		//if ((ship.velocity.length() > 70.0)&&(abs(ship.velocity.angle()-ship.angle))<M_PI/2) {playerResponse.accelerationJets=-1;} 
	
	//	printf("Returning move response: accelerationJets: %lf | rotationJets: %lf\n ", response[j].accelerationJets, response[j].rotationJets);
	}
	}
