#include "AsteroidSpawningSubsystem.h"
#include "Errors.h"
#include "Assert.h"
#include <cmath>

// {{{ AsteroidSpawningSubsystem OK
AsteroidSpawningSubsystem::AsteroidSpawningSubsystem(){
    actualTime=0;   
    nextEventTime=0;
}
// }}}

// {{{ updateTime OK
void AsteroidSpawningSubsystem::updateTime(double timeDelta){
    actualTime+=timeDelta;
    if (actualTime>nextEventTime) {
        asteroidsToSpawn++;
        nextEventTime=nextEventTime+Random::getCenteredDouble(spawnTime,spawnTimeVariability);
    }
}
// }}}

// {{{ init OK
void AsteroidSpawningSubsystem::init(Preferences& preferences){
    asteroidsToSpawn=       preferences.getInt("/asteroid/initialCount");
    spawnTime=              preferences.getDouble("/asteroid/spawnTime");
    spawnTimeVariability=   preferences.getDouble("/asteroid/spawnTimeVariability");
    angularVelocity=           preferences.getDouble("/asteroid/angularSpeed");
    angularVelocityVariability=preferences.getDouble("/asteroid/angularSpeedVariability");
    velocity=               preferences.getDouble("/asteroid/speed");
    velocityVariability=    preferences.getDouble("/asteroid/speedVariability");
    radius=                 preferences.getDouble("/asteroid/radius");
    radiusVariability=
        preferences.getDouble("/asteroid/radiusVariability");
    nextEventTime=spawnTime;
}
// }}}

// {{{ getSpawnCount OK
int AsteroidSpawningSubsystem::getSpawnCount(){
    return asteroidsToSpawn; 
}
// }}}

// {{{ spawn OK
Asteroid* AsteroidSpawningSubsystem::spawn(Point position){
    assert(asteroidsToSpawn>0,"spawning no asteroid!");
    asteroidsToSpawn--;
    Asteroid* ptr=new Asteroid();
    ptr->position=position;
    ptr->angle=Random::getDouble(M_PI);
    ptr->angularVelocity=Random::getCenteredDouble(angularVelocity,angularVelocityVariability);
    double v=Random::getCenteredDouble(velocity,velocityVariability);
    ptr->velocity=Point(cos(ptr->angle)*v,sin(ptr->angle)*v);

    ptr->setRadius(Random::getCenteredDouble(radius,radiusVariability));
    return ptr;
}
// }}}
