#include "EdgeCollider.h"
#include "Preferences.h"

#include <cmath>
#include <MyMath.h>


// {{{ EdgeCollider OK
EdgeCollider::EdgeCollider():edgeDistance(0),edgeForce(0),edgeImpact(0),width(0),height(0){
}
// }}}

// {{{ initialize OK
void EdgeCollider::initialize(const int width, const int height){
    this->width=width;
    this->height=height;
    edgeDistance=Preferences::_getInt("/edgeCollider/distance");
    edgeForce=Preferences::_getDouble("/edgeCollider/force");
    edgeImpact=Preferences::_getDouble("/edgeCollider/impact");
    killDistance=2; // 2 pixels shoud be enough
}
// }}}

// {{{ computeEdgeCollisions OK
void EdgeCollider::computeEdgeCollisions(MassObject* object){
   double x = object->position.x;
   double y = object->position.y;
   double timeDelta=object->getTimeDelta();

   if (x < edgeDistance) {
       Point acceleration = Point(MyMath::sqr(edgeDistance - x) * edgeForce, 0.0);
       object->doImpact(acceleration * timeDelta * edgeImpact, acceleration, 0, 0);
   }

   if (y < edgeDistance) {
       Point acceleration=Point(0.0,MyMath::sqr(edgeDistance - y) * edgeForce);
       object->doImpact(acceleration * timeDelta * edgeImpact, acceleration, 0, 0);
   }

   if (x > width - edgeDistance) {
       Point acceleration = Point( -MyMath::sqr(width - edgeDistance - x) * edgeForce, 0.0);
       object->doImpact(acceleration * timeDelta * edgeImpact, acceleration, 0, 0);
   }

   if (y > height - edgeDistance) {
       Point acceleration = Point(0.0, -MyMath::sqr(height - edgeDistance - y) * edgeForce);
       object->doImpact(acceleration * timeDelta * edgeImpact, acceleration, 0, 0);
   }
}
// }}}


// {{{ edgeKill OK
void EdgeCollider::edgeKill(MassObject* object){
    double x=object->position.x;
    double y=object->position.y;
    if ((x<killDistance) ||
        (y<killDistance) ||
        (x> width - killDistance) ||
        (y> height - killDistance) )
         object->alive=false;
}
// }}}
