#include "Gravity.h"
#include "Errors.h"
#include "Assert.h"
#include "Preferences.h"

// {{{ Gravity OK
Gravity::Gravity():gravitationalKappa(0){
}
// }}}

// {{{ initialize OK
void Gravity::initialize(){
    gravitationalKappa=Preferences::_getDouble("/gravity/kappa");
}
// }}}

// {{{ clear OK
void Gravity::clear(){
    position.clear();
    mass.clear();
    radius.clear();
}
// }}}

// {{{ applyGravityToObject OK
void Gravity::applyGravityToObject(MassObject* object){
    assert(position.size()==mass.size(),"Gravity internal error");
    assert(position.size()==radius.size(),"Gravity internal error");

    const int objects=position.size();
    const double timeDelta=object->getTimeDelta();

    for (int q=0;q<objects;q++){
            Point direction = position[q] - object->position;
            const double d = direction.length();
            const double m = mass[q];
            const double r = radius[q];
              assert(r > 1, "too small radius");

            double velocityImpact;

            if (d >= r) {
                velocityImpact = (timeDelta * gravitationalKappa * m / d / d / d);
            } else {
                velocityImpact = (timeDelta * gravitationalKappa * m  / r / r / r);
            }

            object->doImpact(velocityImpact*direction);
        }
}
// }}}
