#include "PBE_obj.h"

using namespace std;

#include "BasicObject.h"
#include "Ship.h"
#include "Twirler.h"
#include "Exceptions.h"
#include "BlackHole.h"
#include "Nebula.h"
#include "Asteroid.h"
#include "PlasmaBall.h"
#include "CannonBall.h"
#include "Missile.h"

PBE_obj *PBE_obj::po;
float PBE_obj::gtX, PBE_obj::gtY, PBE_obj::ltX, PBE_obj::ltY;
float PBE_obj::pX, PBE_obj::kX, PBE_obj::pY, PBE_obj::kY;
int PBE_obj::world_width, PBE_obj::world_height;

//sok
int PBE_obj::getNextPowerOfTwo(int fold) {
  int pow = 2;
  while (pow < fold) {
    pow *= 2;
  }
  return pow;
}


//sok
bool PBE_obj::isValidTextureIndex(int index) {
	return ((index < PBE_MAX_OBJECT_TYPES) && (index >= 0));
}



//sok
bool PBE_obj::loadTextureFromFile(string sFileName, int texIndex) {
	bool loaded = false;
	
	if (this->isValidTextureIndex(texIndex)) {
		ILuint texid, plachta;
		
		char sBuff[4096];
	
		glEnable(GL_TEXTURE_2D);
	
		ilInit();
		ilGenImages(1, &texid);
		ilBindImage(texid);
		
		
		sprintf(sBuff, "%s", sFileName.c_str());
		if (ilLoadImage(sBuff) && ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE)) {
			//cout << "load_pre[" << texIndex << "]: " << this->texs[texIndex].loaded << endl;
			cout << "[PBE_OGL] Loading sprite (" << sBuff << ")";

			plachta = ilCloneCurImage(); //should be deprecated, but ... don't touch :)
			
			this->texs[texIndex].sizeX = this->getNextPowerOfTwo(ilGetInteger(IL_IMAGE_WIDTH));
			this->texs[texIndex].sizeY = this->getNextPowerOfTwo(ilGetInteger(IL_IMAGE_HEIGHT));
			
			cout << " orig size: " << ilGetInteger(IL_IMAGE_WIDTH) << " x " << ilGetInteger(IL_IMAGE_HEIGHT) << endl;
			
			ILuint ilDepth = ilGetInteger(IL_IMAGE_DEPTH);
			ILuint ilBPP = ilGetInteger(IL_IMAGE_BPP);
			ILuint ilFormat = ilGetInteger(IL_IMAGE_FORMAT);
			ILuint ilType = ilGetInteger(IL_IMAGE_TYPE);
			
			ilBindImage(plachta);
			ilTexImage(
					this->texs[texIndex].sizeX, 
					this->texs[texIndex].sizeY, 
					ilDepth, 
					ilBPP, 
					ilFormat, 
					ilType, 
					NULL);			
			ilClearColour(1.0f, 1.0f, 1.0f, 1.0f);
			ilClearImage();			
			
			ilOverlayImage(texid, 0, 0, 0);

			glGenTextures(1, &this->texs[texIndex].hnd);
			
			// Binding of texture name
		    glBindTexture(GL_TEXTURE_2D, this->texs[texIndex].hnd);
		    		    
		    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); // We will use linear interpolation for magnification filter
		    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); // We will use linear interpolation for minifying filter			
		    glTexImage2D(GL_TEXTURE_2D, 
		    			0, 
		    			ilBPP, 
		    			this->texs[texIndex].sizeX, 
		    			this->texs[texIndex].sizeY, 
		    			0, 
		    			ilFormat, 
		    			GL_UNSIGNED_BYTE,
		    			ilGetData());
		    loaded = true;
			ilDeleteImages(1, &plachta);
			ilDeleteImages(1, &texid);
		} else {
			loaded = false;
		}
		glDisable(GL_TEXTURE_2D);
	
		this->texs[texIndex].loaded = loaded;
		//cout << "load_post[" << texIndex << "]: " << this->texs[texIndex].loaded << endl;
		return this->texs[texIndex].loaded;
		
	} else {
		cout << endl << "[PBE_OGL] lolo, akoze indexy, chapes! " << texIndex << endl;
		return false;
	}
}



//sok
PBE_obj::PBE_obj(int w, int h, int screen_num) {
	cout << endl << PBE_OGL_VERSION << endl;
	cout << "[PBE_OGL] Got world " << w << " x " << h << endl;
	
	po = this;
	
	this->screen_num = screen_num;
	world_width = w;
	world_height = h;
	
	
#ifdef PBE_BENCHMARK_MODE
	this->iBomb = PBE_BENCHMARK_FRAMES;
	cout << "[PBE_OGL] Running benchmark mode - no vulgarisms, they make us slower ..." << endl;
#endif
	
	this->bSpritesLoaded = false;

	for (int j = 0; j < PBE_MAX_OBJECT_TYPES; j++) {
		this->texs[j].loaded = false;
		this->texs[j].hnd = 0;
	}

	this->initGLUTwrapper(0, NULL, PBE_WINDOW_INIT_WIDTH, PBE_WINDOW_INIT_HEIGHT);
	
}


void PBE_obj::calibrateCrop() {
	float sX = glutGet(GLUT_WINDOW_WIDTH), sY = glutGet(GLUT_WINDOW_HEIGHT);

	cout << "[PBE_OGL] calibrating on (" << sX << "x" << sY << ") for world (" << world_width << "x" << world_height << ")" << endl;
	
	
	if ((po->screen_num == PBE_SCREEN_LEFT_TOP) || (po->screen_num == PBE_SCREEN_LEFT_BOTTOM)) {
		ltX = 0; gtX = sX;		
	} else {
		ltX = world_width - sX; gtX = world_width;
	}

	if ((po->screen_num == PBE_SCREEN_LEFT_TOP) || (po->screen_num == PBE_SCREEN_RIGHT_TOP)) {
		ltY = 0; gtY = sY;		
	} else {
		ltY = world_height - sY; gtY = world_height;
	}

	if (po->screen_num == PBE_ONE_SCREEN){
		ltY = 0; gtY = world_height;
		ltX = 0; gtX = world_width;
	}

	
	if ((sX == 1280) && (sY == 1024)) {
		kX = 6.2f / sX;
		kY = 4.96f / sY;
	} else {
		kX = 6.62f / sX;
		kY = 4.96f / sY;		
	}

	if (po->screen_num == PBE_ONE_SCREEN){
		kX = 6.62f / sX * sX / world_width;
		kY = 4.96f / sY * sY / world_height;
	}

	switch (po->screen_num) {
		case PBE_SCREEN_LEFT_TOP:
			pX = - sX/2;
			pY = - sY/2;
			cout << "[PBE_OGL] SCREEN: Left-Top" << endl;
			break;
			
		case PBE_SCREEN_LEFT_BOTTOM:
			pX = - sX/2;
			pY = - world_height + sY/2;
			cout << "[PBE_OGL] SCREEN: Left-Bottom" << endl;
			break;
			
		case PBE_SCREEN_RIGHT_TOP:
			pY = - sY/2;
			pX = - world_width + sX/2;
			cout << "[PBE_OGL] SCREEN: Right-Top" << endl;
			break;
	
		case PBE_SCREEN_RIGHT_BOTTOM:
			pX = - world_width + sX/2;
			pY = - world_height + sY/2;
			cout << "[PBE_OGL] SCREEN: Right-Bottom" << endl;
			break;

		case PBE_ONE_SCREEN:
			pX = - world_width / 2;
			pY = - world_height / 2;
			cout << "[PBE_OGL] SCREEN: One" << endl;
			break;
	}
	
	
	ltY -= PBE_SCREEN_CROP_BORDER;
	ltX -= PBE_SCREEN_CROP_BORDER;
	gtX += PBE_SCREEN_CROP_BORDER;
	gtY += PBE_SCREEN_CROP_BORDER;
	
	cout << "[PBE_OGL] calibration done (" << ltX << "," << gtX <<  ")x(" << ltY << "x" << gtY << ")";
	cout << " X: +" << pX << " ). " << kX;
	cout << " Y: +" << pY << " ). " << kY << endl;
}


//sok
float PBE_obj::fX(float x) {
	return (x + pX) * kX;
}



//sok
float PBE_obj::fY(float y) {
	return (y + pY) * kY;
}



//sok
bool PBE_obj::isVisible(float x, float y) {
	return ((x > ltX) && (x < gtX) && (y > ltY) && (y < gtY));
}




//sok
void PBE_obj::draw(GLfloat x, GLfloat y, GLfloat z, GLfloat width, GLfloat height, int type, GLfloat alpha, GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ) {	
	//tutaj slo prec do ::begin()
	//glMatrixMode(GL_MODELVIEW);
	//glPushMatrix();
	
	
		
	if (isValidTextureIndex(type)
		&& po->texs[type].loaded) {
		
		glLoadIdentity();
		glTranslatef(x, y, z - 6.0f);
		glRotatef(-alpha*180/PI, 0.0f, 0.0f, 1.0f);
		glScalef(scaleX, scaleY, scaleZ);

		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, po->texs[type].hnd);
			glBegin(GL_QUADS);
			    glTexCoord2f(0.0f, 0.0f); glVertex2f(-width/2, -height/2);
			    glTexCoord2f(0.0f, 1.0f); glVertex2f(-width/2, height/2);
			    glTexCoord2f(1.0f, 1.0f); glVertex2f(width/2, height/2);
			    glTexCoord2f(1.0f, 0.0f); glVertex2f(width/2, -height/2);
		    glEnd();
	    glDisable(GL_TEXTURE_2D);
	    
	    
	} else {
		//glDisable(GL_TEXTURE_2D);
		//cout << "[PBE_OGL] non-textured! - .type = " << type << ";   .loaded = " << po->texs[type].loaded << endl;
	}

	
	//prec do ::flip()
	//glPopMatrix();
}



//sok
bool PBE_obj::loadSprites() {
	
	cout << "[PBE_OGL] Loading sprites..." << endl;
	if (this->loadTextureFromFile(string("./sprites/_blackhole_cw.png"), GFX_OBJ_BLACKHOLE_CW)
		&& this->loadTextureFromFile(string("./sprites/_blackhole_ccw.png"), GFX_OBJ_BLACKHOLE_CCW)
		&& this->loadTextureFromFile(string("./sprites/_asteroid.png"), GFX_OBJ_ASTEROID)
		&& this->loadTextureFromFile(string("./sprites/_twirler_cw.png"), GFX_OBJ_TWIRLER_CW)
		&& this->loadTextureFromFile(string("./sprites/_twirler_ccw.png"), GFX_OBJ_TWIRLER_CCW)
		&& this->loadTextureFromFile(string("./sprites/_nebula.png"), GFX_OBJ_NEBULA)
		&& this->loadTextureFromFile(string("./sprites/_explosion.png"), GFX_OBJ_EXPLOSION)
		&& this->loadTextureFromFile(string("./sprites/_plasma.png"), GFX_OBJ_WEAPON_PLASMA)
		&& this->loadTextureFromFile(string("./sprites/_canon.png"), GFX_OBJ_WEAPON_CANON)
		&& this->loadTextureFromFile(string("./sprites/_missile.png"), GFX_OBJ_WEAPON_MISSILE)
		&& this->loadTextureFromFile(string("./sprites/_shipa.png"), GFX_OBJ_SHIPA)
		&& this->loadTextureFromFile(string("./sprites/_shipb.png"), GFX_OBJ_SHIPB)
		&& this->loadTextureFromFile(string("./sprites/_shipc.png"), GFX_OBJ_SHIPC)
		&& this->loadTextureFromFile(string("./sprites/_shipd.png"), GFX_OBJ_SHIPD)
		&& this->loadTextureFromFile(string("./sprites/_shipe.png"), GFX_OBJ_SHIPE)
		&& this->loadTextureFromFile(string("./sprites/background.jpg"), GFX_BACKGROUND_TEXTURE)
		) {
		cout << "[PBE_OGL] PNG sprite(s) naloadovane..." << endl;
		
		return this->bSpritesLoaded = true;

	} else if (this->loadTextureFromFile(string("./sprites/_blackhole_cw.png.gif"), GFX_OBJ_BLACKHOLE_CW)
		&& this->loadTextureFromFile(string("./sprites/_blackhole_ccw.png.gif"), GFX_OBJ_BLACKHOLE_CCW)
		&& this->loadTextureFromFile(string("./sprites/_asteroid.png.gif"), GFX_OBJ_ASTEROID)
		&& this->loadTextureFromFile(string("./sprites/_twirler_cw.png.gif"), GFX_OBJ_TWIRLER_CW)
		&& this->loadTextureFromFile(string("./sprites/_twirler_ccw.png.gif"), GFX_OBJ_TWIRLER_CCW)
		&& this->loadTextureFromFile(string("./sprites/_nebula.png.gif"), GFX_OBJ_NEBULA)
		&& this->loadTextureFromFile(string("./sprites/_explosion.png.gif"), GFX_OBJ_EXPLOSION)
		&& this->loadTextureFromFile(string("./sprites/_plasma.png.gif"), GFX_OBJ_WEAPON_PLASMA)
		&& this->loadTextureFromFile(string("./sprites/_canon.png.gif"), GFX_OBJ_WEAPON_CANON)
		&& this->loadTextureFromFile(string("./sprites/_missile.png.gif"), GFX_OBJ_WEAPON_MISSILE)
		&& this->loadTextureFromFile(string("./sprites/_shipa.png.gif"), GFX_OBJ_SHIPA)
		&& this->loadTextureFromFile(string("./sprites/_shipb.png.gif"), GFX_OBJ_SHIPB)
		&& this->loadTextureFromFile(string("./sprites/_shipc.png.gif"), GFX_OBJ_SHIPC)
		&& this->loadTextureFromFile(string("./sprites/_shipd.png.gif"), GFX_OBJ_SHIPD)
		&& this->loadTextureFromFile(string("./sprites/_shipe.png.gif"), GFX_OBJ_SHIPE)
		&& this->loadTextureFromFile(string("./sprites/background.jpg"), GFX_BACKGROUND_TEXTURE)
		) {
		cout << "[PBE_OGL] fallback GIF sprite(s) naloadovane..." << endl;
		
		return this->bSpritesLoaded = true;

	} else {
		cout << "[PBE_OGL] Nenaloadovalo sprite(s), skusime radsej fantu..." << endl;
		return this->bSpritesLoaded = false;
	}
	
}



//sok
PBE_obj::~PBE_obj() {
	for (int i = 0; i < PBE_MAX_OBJECT_TYPES; i++) {
		if (this->texs[i].loaded)
			glDeleteTextures(1, &this->texs[i].hnd);
	}


	
	// artefakty z glut_wrapper.cpp
    glutDestroyWindow(this->window);
}



//enjoy the ride
void PBE_obj::initGLUTwrapper(int argc, char** argv, int w, int h) {		
    glutInit(&argc, argv);
    glutInitDisplayMode (GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA | GLUT_DEPTH); //GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize (w, h);
    glutInitWindowPosition (0, 0);

    //sprintf(sBuff, "\nKSP 47 Proboj Engine v.%s.%s.%s-%s\n", VER_MAJOR, VER_MINOR, VER_REVISION, VER_COMMENT);
    
    this->window = glutCreateWindow ((char *) sBuff);

    
    // -------------------------------------[init OpenGL ficurii]----------------------------------------
    // Basic setup toho OpenGL frajera :)
    glShadeModel(GL_SMOOTH);
    glClearColor(0.0f, 0.0f, 0.0f, 0.5f);	// cierne pozadie
    glClearDepth(1.0f);						// buffer pamati hlbky

    glEnable(GL_DEPTH_TEST);				// zapni pamat hlbky
    glDepthFunc(GL_LEQUAL);					// typ testu v pamati hlbky
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glEnable(GL_BLEND);


    // -------------------------------------[load objektovych OpenGL ficurii]----------------------------------------
    this->loadSprites();



    // -------------------------------------[callback functions neriesime]----------------------------------------
    //glutDisplayFunc(render);	//renderujeme inak
    //glutIdleFunc(render);		//idle sme 
    //glutReshapeFunc(reshape);
    //glutKeyboardFunc(keyboard);	//keyb input sa neriesi
    //glutMouseFunc(mouse); 		//tu neni vo cem :)

    //glutSpecialFunc(special_keys);



    // -------------------------------------[sem veci z reshape]----------------------------------------
	int www = glutGet(GLUT_SCREEN_WIDTH), hhh = glutGet(GLUT_SCREEN_HEIGHT);
    
    glViewport(0, 0, www, hhh);
    glutFullScreen();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, (float) www/(float) hhh, 1.0, 5000.0);
    glMatrixMode(GL_MODELVIEW);

    PBE_obj::calibrateCrop();
    
    // -------------------------------------[a tento si moze piskat]----------------------------------------
    //glutMainLoop();
}





void PBE_obj::begin() {
#ifdef PBE_BENCHMARK_MODE
	if (this->iBomb > 0) { 
		if (this->iBomb-- % PBE_BENCHMARK_MODULO == 0) 
			cout << "[PBE_OGL] Frames left: " << this->iBomb << endl;
	} else {
		exit(23);
	}
#endif

	// -------------------------------------[sem veci zo zaciatku render]----------------------------------------
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    
    
    // -------------------------------------[sem veci zo zaciatku draw]----------------------------------------
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	//test fucker
	draw(	fX(world_width / 2),
			fY(world_height / 2), 
			-1.0f,
			(world_width+100)*kX, 
			(world_height+100)*kY, 
			GFX_BACKGROUND_TEXTURE, 
			0.0f, 
			1.0f, 
			1.0f, 
			1.0f);	
}



void PBE_obj::flip() {

	// -------------------------------------[sem veci z konca draw :)]----------------------------------------
	glPopMatrix();



	// -------------------------------------[sem veci z konca render :)]----------------------------------------
    glutSwapBuffers();
}

void PBE_obj::drawBasicObject(BasicObject* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*2*kX, 
			object->radius*2*kY,
			GFX_OBJ_EXPLOSION,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}


void PBE_obj::drawShip(Ship* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*4*kX, 
			object->radius*4*kY,
			GFX_OBJ_SHIPA + object->player,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}

void PBE_obj::drawAsteroid(Asteroid* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*2*kX, 
			object->radius*2*kY,
			GFX_OBJ_ASTEROID,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}

void PBE_obj::drawTwirler(Twirler* object) {
	if (isVisible(object->position.x, object->position.y))
		po->draw(
				fX(object->position.x), 
				fY(object->position.y), 
				0.0f, 
				object->radius*2*kX, 
				object->radius*2*kY,
				(object->clockwise == 0 ? GFX_OBJ_TWIRLER_CCW : GFX_OBJ_TWIRLER_CW),
				object->angle,
				1.0f,
				1.0f,
				1.0f);
}

void PBE_obj::drawBlackHole(BlackHole* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*2*kX, 
			object->radius*2*kY,
			(object->angularVelocity > 0 ? GFX_OBJ_BLACKHOLE_CCW : GFX_OBJ_BLACKHOLE_CW),
			object->angle,
			1.0f,
			1.0f,
			1.0f);		
}

void PBE_obj::drawNebula(Nebula* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.1f, 
			object->radius*2*kX, 
			object->radius*2*kY,
			GFX_OBJ_NEBULA,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}

void PBE_obj::drawMissile(Missile* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*2*kX, 
			object->radius*2*kY,
			GFX_OBJ_WEAPON_MISSILE,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}

void PBE_obj::drawPlasmaBall(PlasmaBall* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*3*kX, 
			object->radius*3*kY,
			GFX_OBJ_WEAPON_PLASMA,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}

void PBE_obj::drawCannonBall(CannonBall* object) {
	if (isVisible(object->position.x, object->position.y))
	po->draw(
			fX(object->position.x), 
			fY(object->position.y), 
			0.0f, 
			object->radius*4*kX, 
			object->radius*4*kY,
			GFX_OBJ_WEAPON_CANON,
			object->angle,
			1.0f,
			1.0f,
			1.0f);	
}

void PBE_obj::_drawObject(BasicObject* object, int R, int G, int B) {
	cout << "[PBE_OGL] muhaha" << endl;
	//this->draw(x,y,z,width, height,GFX_OBJ_FORK, alpha, scaleX, scaleY, scaleZ);
}




