#include "PlasmaBall.h"
#include "Types.h"
#include "World.h"
#include "Explosion.h"
#include "Serializer.h"
#include "Preferences.h"
#include "PBE_obj.h"

double PlasmaBall::detonatingImpact;
double PlasmaBall::detonatingDamage;
double PlasmaBall::detonatingPlasmaToPlasmaDamage;

double PlasmaBall::defaultRadius;
double PlasmaBall::defaultMass;
double PlasmaBall::defaultAngularMass;

// {{{ initialize OK
void PlasmaBall::initialize(){
    detonatingImpact=Preferences::_getDouble("/plasmaBall/detonatingImpact");
    detonatingDamage=Preferences::_getDouble("/plasmaBall/detonatingDamage");
    detonatingPlasmaToPlasmaDamage=
         Preferences::_getDouble("/plasmaBall/detonatingPlasmaToPlasmaDamage");
    defaultRadius=Preferences::_getDouble("/plasmaBall/radius");
    defaultAngularMass=Preferences::_getDouble("/plasmaBall/rotationalMass");
    defaultMass=Preferences::_getDouble("/plasmaBall/mass");
}
// }}}

// {{{ getType OK
int PlasmaBall::getType(){
    return Types::plasmaBall;
}
// }}}

// {{{ doDamage OK
void PlasmaBall::doDamage(double impactDamage, double explosionDamage,  int from, World* world){
    Weapon::doDamage(impactDamage,explosionDamage,from,world);
    double
        pureImpactDamage=impactDamage-explosionDamage*Explosion::impactFactor;
    if (pureImpactDamage>detonatingImpact) {
        this->alive=false; // self kill
        explosionVelocity=velocity;
    }
}
// }}}

// {{{ destroy OK
void PlasmaBall::destroy(World* world){
    if (damage==0) return; // do nothing

    Explosion* ptr=world->addExplosion(this->position,damage);
    ptr->owner=this->owner;
    ptr->velocity=explosionVelocity;
    ptr->timeNotify(0);
}
// }}}

// {{{ PlasmaBall OK
PlasmaBall::PlasmaBall():Weapon(){
    radius=defaultRadius;
    mass=defaultMass;
    angularMass = defaultAngularMass;
    explosionVelocity=Point();

    damage=detonatingDamage;
}
// }}}

// {{{ ~PlasmaBall OK
PlasmaBall::~PlasmaBall(){
}
// }}}

// {{{ doPlasmaCollision OK
void PlasmaBall::doPlasmaCollision(PlasmaBall* another){
    this->alive=false;
    another->alive=false;
    another->damage=0;
    this->damage=detonatingPlasmaToPlasmaDamage;
}
// }}}

// {{{ serialize OK
void PlasmaBall::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeChar(destination,getType());
    Weapon::_serialize(destination,this);
    Serializer::serializeFloat(destination,damage);
}
// }}}

// {{{ deserialize OK
PlasmaBall* PlasmaBall::deserialize(std::vector<unsigned
        char>::iterator& it) {
    PlasmaBall* ptr=new PlasmaBall();
    Weapon::_deserialize(it,ptr);
    ptr->damage=Serializer::deserializeFloat(it);
    return ptr;
}
// }}}

// {{{ draw
void PlasmaBall::draw(){
    PBE_obj::drawPlasmaBall(this);
}
// }}}
