#include "PlasmaWeaponSlot.h"

#include "Preferences.h"
#include "Utils.h"
#include "PlasmaBall.h"
#include "World.h"
#include "Serializer.h"
#include "Types.h"
#include <cmath>

double PlasmaWeaponSlot::reloadTime;
double PlasmaWeaponSlot::maximumEnergy;
double PlasmaWeaponSlot::shootEnergy;

double PlasmaWeaponSlot::minAngle;
double PlasmaWeaponSlot::maxAngle;
double PlasmaWeaponSlot::minSpeed;
double PlasmaWeaponSlot::maxSpeed;

// {{{ getReloadTime
double PlasmaWeaponSlot::getReloadTime(){
    return reloadTime;
}
// }}}

// {{{ getTimeToNextShoot()
double PlasmaWeaponSlot::getTimeToNextShoot(){
    return std::max(reload,shootEnergy-energy); 
}
// }}}

// {{{ getNumberOfRemainingShoots
double PlasmaWeaponSlot::getNumberOfRemainingShoots(){
    return (double)((int)(energy/shootEnergy));
}
// }}}

// {{{ getMaxEnergy()
double PlasmaWeaponSlot::getMaxEnergy(){
    return maximumEnergy;
}
// }}}

// {{{ getShootEnergy()
double PlasmaWeaponSlot::getShootEnergy(){
    return shootEnergy;
}
// }}}

// {{{ getActualEnergy()
double PlasmaWeaponSlot::getActualEnergy(){
    return energy;
}
// }}}

// {{{ getMinAngle()
double PlasmaWeaponSlot::getMinAngle(){
    return minAngle;
}
//  }}}
 
// {{{ getMaxAngle()
double PlasmaWeaponSlot::getMaxAngle(){
    return maxAngle;
}
// }}}
 
// {{{ getMinSpeed()
double PlasmaWeaponSlot::getMinSpeed(){
    return minSpeed;
}
// }}} 

// {{{ getMaxSpeed()
double PlasmaWeaponSlot::getMaxSpeed(){
    return maxSpeed;
}
// }}}

// {{{ clone OK
WeaponSlot* PlasmaWeaponSlot::clone(){
    PlasmaWeaponSlot* tmp=new PlasmaWeaponSlot();
    (*tmp)=(*this);
    return tmp;
}
// }}}

// {{{ initialize OK
void PlasmaWeaponSlot::initialize(){
    reloadTime=Preferences::_getDouble("/weapons/plasma/reloadTime");
    maximumEnergy=Preferences::_getDouble("/weapons/plasma/maximumEnergy");
    shootEnergy=Preferences::_getDouble("/weapons/plasma/shootEnergy");

    minAngle=Preferences::_getDouble("/weapons/plasma/minAngle");
    maxAngle=Preferences::_getDouble("/weapons/plasma/maxAngle");
    minSpeed=Preferences::_getDouble("/weapons/plasma/minSpeed");
    maxSpeed=Preferences::_getDouble("/weapons/plasma/maxSpeed");
}
// }}}

// {{{ PlasmaWeaponSlot OK
PlasmaWeaponSlot::PlasmaWeaponSlot(){
    reload=0;
    energy=0;
}
// }}}

// {{{ ~PlasmaWeaponSlot OK
PlasmaWeaponSlot::~PlasmaWeaponSlot(){

}
// }}}

// {{{ timeNotify OK
void PlasmaWeaponSlot::timeNotify(double timeDelta){
    reload-=timeDelta;
    if (reload<0) reload=0;

    energy+=timeDelta;
    if (energy>maximumEnergy) energy=maximumEnergy;
}
// }}}

// {{{ shoot OK
void PlasmaWeaponSlot::shoot(Ship* ship, double angle, double speed,
        double additionalInfo, World* world){
    angle=Utils::normalizeAngle(angle);
    if (angle<minAngle) return;
    if (angle>maxAngle) return;
    if (speed<minSpeed) return;
    if (speed>maxSpeed) return;

    if (energy<shootEnergy) return;
    if (reload>0) return;

    energy-=shootEnergy;
    reload=reloadTime;


    PlasmaBall* ptr=new PlasmaBall();
    angle+=ship->angle;

    double dist=ship->radius+10;
    Point p=Point(cos(angle),sin(angle));

    ptr->position = ship->position + p*dist;
    ptr->velocity = ship->velocity+p*speed;
    ptr->owner = ship->owner;
    ptr->clean();

    world->addLightObject(ptr);
    world->communication.shootPlasma();
}
// }}}

// {{{ serialize OK
void PlasmaWeaponSlot::serialize(std::vector<unsigned char>&
        destination){
    Serializer::serializeChar(destination,Types::plasmaSlot);
    Serializer::serializeFloat(destination,reload);
    Serializer::serializeFloat(destination,energy);
}
// }}}

// {{{ deserialize OK
PlasmaWeaponSlot* PlasmaWeaponSlot::deserialize(std::vector<unsigned
        char>::iterator& it){
    PlasmaWeaponSlot* ptr=new PlasmaWeaponSlot();
    ptr->reload=Serializer::deserializeFloat(it);
    ptr->energy=Serializer::deserializeFloat(it);
    return ptr;
}
// }}}
