#include "Point.h"
#include <cmath>
#include <algorithm>
#include "Serializer.h"
#include "MyStringUtils.h"
#include "Errors.h"
#include "Assert.h"

// {{{ serialize OK
void Point::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeFloat(destination, (float) x);
    Serializer::serializeFloat(destination, (float) y); 
}
// }}}

// {{{ deserialize OK
Point Point::deserialize(std::vector<unsigned char>::iterator& it){
    Point p;
    p.x=Serializer::deserializeFloat(it);
    p.y=Serializer::deserializeFloat(it);
    return p;
}
// }}}

// {{{ normalize OK
void Point::normalize(){
    if (this->length()<1e-10) return; 
    (*this)/=this->length();
}
// }}}

// {{{ getPointFromString
Point Point::getPointFromString(const std::string& str){
    std::vector<std::string> tmp=
        MyStringUtils::splitString(str,",");
    assert(tmp.size()==2,"can't convert string to point");
    Point res;
    res.x=MyStringUtils::stringToDouble(tmp[0]);
    res.y=MyStringUtils::stringToDouble(tmp[1]);
    return res;
}
// }}}
