#include "Proboj.h"
#include "Macro.h"


#include "Errors.h"
#include "Exceptions.h"
#include "Preferences.h"
#include "Engine.h"

#include <iostream>
#include <SDL.h>

#define LOG_FILE "proboj.log"
#define INI_FILE "proboj.ini"

ProbojEngine engine;

// {{{ init OK
int init(){
    Errors::createInstance(LOG_FILE);
    Preferences::createInstance();
    Preferences::_loadFromFile(INI_FILE);

    if (SDL_Init(SDL_INIT_TIMER)<0){
        Errors::_addError("Can't init SDL",Errors::ERROR);
        return 0;
    }
    Errors::_addError("SDL init done",Errors::NOTICE);

    return 1;
}
// }}}

// {{{ finalize OK
void finalize(){
    Preferences::destroyInstance();
    Errors::destroyInstance();
}
// }}}

// {{{ main OK
int main(){
	std::cout << "lolo starting up" << std::endl;
    try {

        if (init()) {
            engine.initializeGame("mapa.dat");
            engine.runGame();
        }

        finalize();
    } catch (Exception &e){
        printf("There was an exception %s %s\n",
                e.what(),e.getStringStack().c_str());
    } catch (std::exception &e){
        printf("There was an exception %s\n",e.what());
    }
}
// }}}
