#include "Serializer.h"
#include "Macro.h"
#include "Errors.h"
#include "Assert.h"
#include "Types.h"
#include "MassObject.h"
#include "Ship.h"
#include "BlackHole.h"
#include "Asteroid.h"
#include "Twirler.h"
#include "Nebula.h"
#include "CannonBall.h"
#include "PlasmaBall.h"
#include "Missile.h"
#include "Explosion.h"


// {{{ checkTypes OK
void Serializer::checkTypes(){
    int i;
    double d;
    float f;
    assert(intSize==sizeof(i),"int size mismatch");
    assert(doubleSize==sizeof(d),"double size mismatch");
    assert(floatSize==sizeof(f),"float size mismatch");
}
// }}}

// {{{ serializeChar OK
void Serializer::serializeChar(std::vector<unsigned char>& destination,
        const char value){
   unsigned char *ptr=(unsigned char*) &value;
   FOR(q,charSize){
       destination.push_back(*ptr);
       ++ptr;
   }
}
// }}}

// {{{ serializeInt OK
void Serializer::serializeInt(std::vector<unsigned char>& destination,
        const int value){
   unsigned char *ptr=(unsigned char*) &value;
   FOR(q,intSize){
       destination.push_back(*ptr);
       ++ptr;
   }
}
// }}}

// {{{ serializeDouble OK
void Serializer::serializeDouble(std::vector<unsigned char>& destination,
        const double value){
   unsigned char *ptr=(unsigned char*) &value;
   FOR(q,doubleSize){
       destination.push_back(*ptr);
       ++ptr;
   }
}
// }}}

// {{{ serializeFloat OK
void Serializer::serializeFloat(std::vector<unsigned char>& destination,
        const float value){
   unsigned char *ptr=(unsigned char*) &value;
   FOR(q,floatSize){
       destination.push_back(*ptr);
       ++ptr;
   }
}
// }}}

// {{{ deserializeChar OK
char Serializer::deserializeChar(std::vector<unsigned char>::iterator& it){
    char result;
    unsigned char*ptr=(unsigned char*) &result;
    FOR(q,charSize){
        (*ptr)=(*it);
        ++ptr;
        ++it;
    }
    return result;
}
// }}}

// {{{ deserializeInt OK
int Serializer::deserializeInt(std::vector<unsigned char>::iterator& it){
    int result;
    unsigned char*ptr=(unsigned char*) &result;
    FOR(q,intSize){
        (*ptr)=(*it);
        ++ptr;
        ++it;
    }
    return result;
}
// }}}

// {{{ deserializeDouble OK
double Serializer::deserializeDouble(std::vector<unsigned char>::iterator& it){
    double result;
    unsigned char*ptr=(unsigned char*) &result;
    FOR(q,doubleSize){
        (*ptr)=(*it);
        ++ptr;
        ++it;
    }
    return result;
}
// }}}

// {{{ deserializeFloat OK
float Serializer::deserializeFloat(std::vector<unsigned char>::iterator& it){
    float result;
    unsigned char*ptr=(unsigned char*)&result;
    FOR(q,floatSize){
        (*ptr)=(*it);
        ++ptr;
        ++it;
    }
    return result;
}
// }}}

// {{{ serializePoint OK
void Serializer::serializePoint(std::vector<unsigned
        char>& destination, Point value){
    value.serialize(destination);
}
// }}}

// {{{ deserializePoint OK
Point Serializer::deserializePoint(std::vector<unsigned
        char>::iterator& it){
    return Point::deserialize(it);
}
// }}}

//TODO
// {{{ deserializeObject
BasicObject* Serializer::deserializeObject(std::vector<unsigned
        char>::iterator& it){
    int type=deserializeChar(it);
    //fprintf(stderr,"deserialize object of type %d \n",type);
    if (type==Types::ship) return Ship::deserialize(it);
    if (type==Types::blackHole) return BlackHole::deserialize(it);
    if (type==Types::asteroid) return Asteroid::deserialize(it);

    if (type==Types::plasmaBall) return PlasmaBall::deserialize(it);
    if (type==Types::cannonBall) return CannonBall::deserialize(it);
    if (type==Types::missile) return Missile::deserialize(it);

    if (type==Types::explosion) return Explosion::deserialize(it);
    if (type==Types::nebula) return Nebula::deserialize(it);
    if (type==Types::twirler) return Twirler::deserialize(it);
    return NULL;

}
// }}}
