#include "CannonBall.h"
#include "Types.h"
#include "Preferences.h"
#include "Serializer.h"
#include "World.h"
#include "ObjectGraphics.h"

double CannonBall::defaultMass;
double CannonBall::defaultRadius;
double CannonBall::durability;

// {{{ initialize OK
void CannonBall::initialize(){
    defaultMass=Preferences::_getDouble("/cannonBall/defaultMass");
    defaultRadius=Preferences::_getDouble("/cannonBall/defaultRadius");    
    durability=Preferences::_getDouble("/cannonBall/durability");
}
// }}}

// {{{ getType OK
int CannonBall::getType(){
    return Types::cannonBall;
}
// }}}

// {{{ CannonBall OK
CannonBall::CannonBall():Weapon(){
    mass=defaultMass;
    radius=defaultRadius;
    angularMass=0.5*radius*radius*mass;
    angularVelocity=0;
}
// }}}

// {{{ ~CannonBall OK
CannonBall::~CannonBall(){

}
// }}}

// {{{ serialize OK
void CannonBall::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeChar(destination,getType());
    Weapon::_serialize(destination,this);
}
// }}}

// {{{ deserialize OK
CannonBall* CannonBall::deserialize(std::vector<unsigned char>::iterator&
        it){
    CannonBall* ptr=new CannonBall();
    Weapon::_deserialize(it,ptr);
    return ptr;

}
// }}}


// {{{ doDamage
void CannonBall::doDamage(double impactDamage, double explosionDamage,
        int from,World* world){
    if (impactDamage+explosionDamage > durability) alive=false;
}
// }}}

// {{{ draw
void CannonBall::draw(){
    ObjectGraphics::drawCannonBall(this);
}
// }}}
