#include "CannonWeaponSlot.h"
#include "World.h"
#include "Ship.h"
#include "Utils.h"
#include "CannonBall.h"
#include "Preferences.h"
#include "Macro.h"
#include "Types.h"
#include "Serializer.h"

double CannonWeaponSlot::reloadTime;
int CannonWeaponSlot::minCount;
int CannonWeaponSlot::maxCount;
double CannonWeaponSlot::scatterAngle;
double CannonWeaponSlot::cannonBallSpeed;
double CannonWeaponSlot::shootDistance;
double CannonWeaponSlot::recoilFactor;
double CannonWeaponSlot::distanceVariablity;

// {{{ getReloadTime
double CannonWeaponSlot::getReloadTime(){
    return reloadTime;
}
// }}}

// {{{ getTimeToNextShoot()
double CannonWeaponSlot::getTimeToNextShoot(){
    return reload;
}
// }}}

// {{{ getNumberOfRemainingShoots()
double CannonWeaponSlot::getNumberOfRemainingShoots(){
    return maxCount;
}
// }}}

// {{{ getMaxEnergy
double CannonWeaponSlot::getMaxEnergy(){
    return maxCount*reloadTime;
}
// }}}

// {{{ getShootEnergy
double CannonWeaponSlot::getShootEnergy(){
    return reloadTime;
}
// }}}

// {{{ getActualEnergy
double CannonWeaponSlot::getActualEnergy(){
    return getMaxEnergy()-reload;
}
// }}}

// {{{ getMinAngle
double CannonWeaponSlot::getMinAngle(){
    return 0;
}
// }}} 

// {{{ getMaxAngle
double CannonWeaponSlot::getMaxAngle(){
    return 0;
}
// }}}

// {{{ getMinSpeed 
double CannonWeaponSlot::getMinSpeed(){
    return 0;
}
// }}}

// {{{ getMaxSpeed()
double CannonWeaponSlot::getMaxSpeed(){
    return cannonBallSpeed;
}
// }}}

// {{{ CannonWeaponSlot OK
CannonWeaponSlot::CannonWeaponSlot(){
    reload=0;
}
// }}}

// {{{ ~CannonWeaponSlot OK
CannonWeaponSlot::~CannonWeaponSlot(){

}
// }}}

// {{{ clone OK
WeaponSlot* CannonWeaponSlot::clone(){
    CannonWeaponSlot* ptr=new CannonWeaponSlot();
    (*ptr)=(*this);
    return ptr;
}
// }}}

// {{{ shoot OK
void CannonWeaponSlot::shoot(Ship* ship, double angle, double speed,
            double additionalInfo, World* world){
    if (angle!=0) return;
    if (speed!=cannonBallSpeed) return;
    int cnt=(int) additionalInfo;
    if (cnt<minCount || cnt>maxCount) return;
    if (reload>0) return;

    FOR(q,cnt) {
        CannonBall* ptr=new CannonBall;
        double xangle=
            Random::getCenteredDouble(ship->angle,scatterAngle);
        Point direction=Point(cos(xangle),sin(xangle));
        ptr->velocity=direction*cannonBallSpeed+ship->velocity;
        ptr->position=ship->position+direction*(shootDistance+Random::getDouble(distanceVariablity));
        ptr->owner=ship->player;
        ptr->timeNotify(ship->getTimeDelta());
        ptr->clean();
        ship->doImpact(-direction*cannonBallSpeed*ptr->mass/ship->mass*ship->getTimeDelta()*recoilFactor);
        world->addLightObject(ptr);

        reload+=reloadTime;
    }
    world->communication.shootCannon();

}
// }}}

// {{{ initialize OK
void CannonWeaponSlot::initialize(){
    shootDistance=15;
    distanceVariablity=15;
    minCount=Preferences::_getInt("/weapons/cannon/minCount");
    maxCount=Preferences::_getInt("/weapons/cannon/maxCount");
    reloadTime=Preferences::_getDouble("/weapons/cannon/reloadTime");
    scatterAngle=Preferences::_getDouble("/weapons/cannon/scatterAngle");
    cannonBallSpeed=Preferences::_getDouble("/weapons/cannon/speed");
    recoilFactor=Preferences::_getDouble("/weapons/cannon/recoilFactor");
}
// }}} 

// {{{ timeNotify OK
void CannonWeaponSlot::timeNotify(double timeDelta){
    reload-=timeDelta;
    if (reload<0) reload=0;
}
// }}}

// {{{ serialize OK
void CannonWeaponSlot::serialize(std::vector<unsigned char>&
        destination){
    Serializer::serializeChar(destination,Types::cannonSlot);
    Serializer::serializeFloat(destination,reload);
}
// }}}

// {{{ deserialize OK
CannonWeaponSlot* CannonWeaponSlot::deserialize(std::vector<unsigned
        char>::iterator& it){
    CannonWeaponSlot* ptr=new CannonWeaponSlot();
    ptr->reload=Serializer::deserializeFloat(it);
    return ptr;
}
// }}}
