#include "CollisionSolver.h"
#include "World.h"
#include "Types.h"
#include "Preferences.h"
#include "PlasmaBall.h"
#include "Errors.h"


double CollisionSolver::COLLISION_SKIP;
double CollisionSolver::COLLISION_BOUNCE;
double CollisionSolver::impactToDamage;

// {{{ initialize OK
void CollisionSolver::initialize(){
   COLLISION_SKIP=Preferences::_getDouble("/collisionSolver/collisionSkip"); 
   COLLISION_BOUNCE=Preferences::_getDouble("/collisionSolver/collisionBounce");
   impactToDamage=Preferences::_getDouble("/collisionSolver/impactToDamage");

}
// }}}

//TODO
// {{{ doMassMassCollision 
void CollisionSolver::doMassMassCollision(MassObject* object1, MassObject*
        object2, World* world){

    int t1=object1->getType();
    int t2=object2->getType();

    if (t1==Types::blackHole && t2==Types::blackHole) return;
    if (t1==Types::blackHole) {
        object2->doDamage(INFINITE_DAMAGE,0,-1,world);
    }

    if (t2==Types::blackHole) {
        object1->doDamage(INFINITE_DAMAGE,0,-1,world);
    }

    if (t1==Types::plasmaBall && t2==Types::plasmaBall) {
        PlasmaBall* ptr1=dynamic_cast<PlasmaBall*>(object1);
        PlasmaBall* ptr2=dynamic_cast<PlasmaBall*>(object2);
        ptr1->doPlasmaCollision(ptr2);
        world->communication.plasmaToPlasmaExplode();
        return;
    }


    
    Point v = (object1->velocity * object1->mass + 
               object2->velocity * object2->mass)/
              (object1->mass + object2->mass);
    Point v1=v-object1->velocity;
    Point v2=v-object2->velocity;

/*
    if (v1.length()>250 || v2.length()>250) {
        fprintf(stderr,"%lf %lf   %lf %lf %d %d\n",
                object1->velocity.x,object1->velocity.y,
                object1->position.x,object1->position.y,
                object1->getType(),object1->id);
        fprintf(stderr,"%lf %lf   %lf %lf %d %d\n",
                object2->velocity.x,object2->velocity.y,
                object2->position.x,object2->position.y,
                object2->getType(),object2->id);
        throw 1; //Exception("xx");


    }
    fprintf(stderr,"%lf %lf %d %d  %lf\n",object1->velocity.length(),
            object2->velocity.length(),object1->id,object2->id,
            (object1->position-object2->position).length());
            */

    Point p=(object1->position-object2->position);
    p.normalize();

    double rot= (object1->angularVelocity * object1->angularMass -
                 object2->angularVelocity * object2->angularMass) /
                (object1->angularMass + object2->angularMass);
    double r1=rot-object1->angularVelocity;
    double r2=-rot-object2->angularVelocity;
    

    double timeDelta=object1->getTimeDelta();
    assert(object2->getTimeDelta()==timeDelta,"wrong time delta");
    
    object1->doDamage(impactToDamage*v1.length()*COLLISION_BOUNCE,-1, object2->owner,
            world);
    object2->doDamage(impactToDamage*v2.length()*COLLISION_BOUNCE,-1, object1->owner,
            world);

    object1->doImpact(p*timeDelta*COLLISION_SKIP,v1*COLLISION_BOUNCE,0,r1);
    object2->doImpact(-p*timeDelta*COLLISION_SKIP,v2*COLLISION_BOUNCE,0,r2);

    
    //object2->alive=false; // kill one of them
}
// }}}

// {{{ doMassMasslessCollision OK
void CollisionSolver::doMassMasslessCollision(MassObject* object1,
        BasicObject* object2, World* world){
    Affector* affector=dynamic_cast<Affector*>(object2);
     affector->affect(object1,world);

}
// }}}
