#include "MissileWeaponSlot.h"

#include "Preferences.h"
#include "Utils.h"
#include "Missile.h"
#include "World.h"
#include "Serializer.h"
#include "Types.h"
#include <cmath>

double MissileWeaponSlot::reloadTime;
double MissileWeaponSlot::maximumEnergy;
double MissileWeaponSlot::shootEnergy;

double MissileWeaponSlot::minAngle;
double MissileWeaponSlot::maxAngle;
double MissileWeaponSlot::minSpeed;
double MissileWeaponSlot::maxSpeed;

// {{{ getReloadTime
double MissileWeaponSlot::getReloadTime(){
    return reloadTime;
}
// }}}

// {{{ getTimeToNextShoot()
double MissileWeaponSlot::getTimeToNextShoot(){
    return std::max(reload,shootEnergy-energy); 
}
// }}}

// {{{ getNumberOfRemainingShoots
double MissileWeaponSlot::getNumberOfRemainingShoots(){
    return (double)((int)(energy/shootEnergy));
}
// }}}

// {{{ getMaxEnergy()
double MissileWeaponSlot::getMaxEnergy(){
    return maximumEnergy;
}
// }}}

// {{{ getShootEnergy()
double MissileWeaponSlot::getShootEnergy(){
    return shootEnergy;
}
// }}}

// {{{ getActualEnergy()
double MissileWeaponSlot::getActualEnergy(){
    return energy;
}
// }}}

// {{{ getMinAngle()
double MissileWeaponSlot::getMinAngle(){
    return minAngle;
}
// }}}

// {{{ getMaxAngle()
double MissileWeaponSlot::getMaxAngle(){
    return maxAngle;
}
// }}}


// {{{ getMinSpeed()
double MissileWeaponSlot::getMinSpeed(){
    return minSpeed;
}
// }}}

// {{{ getMinSpeed()
double MissileWeaponSlot::getMaxSpeed(){
    return maxSpeed;
}
// }}}

// {{{ clone OK
WeaponSlot* MissileWeaponSlot::clone(){
    MissileWeaponSlot* tmp=new MissileWeaponSlot();
    (*tmp)=(*this);
    return tmp;
}
// }}}

// {{{ initialize OK
void MissileWeaponSlot::initialize(){
    reloadTime=Preferences::_getDouble("/weapons/missile/reloadTime");
    maximumEnergy=Preferences::_getDouble("/weapons/missile/maximumEnergy");
    shootEnergy=Preferences::_getDouble("/weapons/missile/shootEnergy");

    minAngle=Preferences::_getDouble("/weapons/missile/minAngle");
    maxAngle=Preferences::_getDouble("/weapons/missile/maxAngle");
    minSpeed=Preferences::_getDouble("/weapons/missile/minSpeed");
    maxSpeed=Preferences::_getDouble("/weapons/missile/maxSpeed");
}
// }}}

// {{{ MissileWeaponSlot OK
MissileWeaponSlot::MissileWeaponSlot(){
    reload=0;
    energy=0;
}
// }}}

// {{{ ~MissileWeaponSlot OK
MissileWeaponSlot::~MissileWeaponSlot(){

}
// }}}

// {{{ timeNotify OK
void MissileWeaponSlot::timeNotify(double timeDelta){
    reload-=timeDelta;
    if (reload<0) reload=0;

    energy+=timeDelta;
    if (energy>maximumEnergy) energy=maximumEnergy;
}
// }}}

// {{{ shoot OK
void MissileWeaponSlot::shoot(Ship* ship, double angle, double speed,
        double additionalInfo, World* world){
    angle=Utils::normalizeAngle(angle);
    if (angle<minAngle) return;
    if (angle>maxAngle) return;
    if (speed<minSpeed) return;
    if (speed>maxSpeed) return;

    if (energy<shootEnergy) return;
    if (reload>0) return;

    energy-=shootEnergy;
    reload=reloadTime;


    Missile* ptr=new Missile(world);
    angle+=ship->angle;

    double dist=ship->radius+10;
    Point p=Point(cos(angle),sin(angle));

    ptr->position = ship->position + p*dist;
    ptr->velocity = ship->velocity+p*speed;
    ptr->owner = ship->owner;
    ptr->angle = ship->angle;
    ptr->clean();

    world->addLightObject(ptr);
    world->communication.shootMissile();
}
// }}}

// {{{ serialize OK
void MissileWeaponSlot::serialize(std::vector<unsigned char>&
        destination){
    Serializer::serializeChar(destination,Types::missileSlot);
    Serializer::serializeFloat(destination,reload);
    Serializer::serializeFloat(destination,energy);
}
// }}}

// {{{ deserialize OK
MissileWeaponSlot* MissileWeaponSlot::deserialize(std::vector<unsigned
        char>::iterator& it){
    MissileWeaponSlot* ptr=new MissileWeaponSlot();
    ptr->reload=Serializer::deserializeFloat(it);
    ptr->energy=Serializer::deserializeFloat(it);
    return ptr;
}
// }}}
