#include <cmath>

#include "Twirler.h"
#include "MassObject.h"
#include "World.h"
#include "ObjectGraphics.h"
#include "Types.h"
#include "Serializer.h"
#include "Random.h"
// {{{ getType
int Twirler::getType(){
    return Types::twirler;
}
// }}}

// {{{ affect
void Twirler::affect(MassObject* object, World* world){
    double timeDelta=object->getTimeDelta();
    double d=(clockwise)?1:-1;

    double l=(this->position-object->position).length();
    if (l>radius) l=radius;
    double max=maxSpin*(radius-l)/radius;
    if (object->angularVelocity*d < max) {
        object->doImpact(Point(0,0),d*spinPerSecond*timeDelta/object->mass);
    }
}
// }}}

// {{{ Twirler
Twirler::Twirler():Affector(){
    maxSpin=0;
    spinPerSecond=0;
    clockwise=0;
    angle=Random::getDouble(2*M_PI);
}
// }}}

// {{{ ~Twirler
Twirler::~Twirler(){

}
// }}}

// {{{ draw
void Twirler::draw(){
    ObjectGraphics::drawTwirler(this);
}
// }}}

// {{{ serialize
void Twirler::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeChar(destination,getType());
    BasicObject::_serialize(destination,this);
    Serializer::serializeFloat(destination,maxSpin);
    Serializer::serializeFloat(destination,spinPerSecond);
    Serializer::serializeChar(destination,clockwise);
}
// }}}

// {{{ deserialize
Twirler* Twirler::deserialize(std::vector<unsigned char>::iterator&
        it){
    Twirler* ptr=new Twirler();
    BasicObject::_deserialize(it,ptr);
    ptr->maxSpin=Serializer::deserializeFloat(it);
    ptr->spinPerSecond=Serializer::deserializeFloat(it);
    ptr->clockwise=Serializer::deserializeChar(it);
    return ptr;
}
// }}}

void Twirler::timeNotify(double timeDelta){
    Affector::timeNotify(timeDelta);
    angle+=timeDelta*spinPerSecond/10*(clockwise*2-1);

}
